@@ -1,7 +1,8 @@
 /* directory.c
  *
- * Copyright  2001 Lutz Mller <lutz@users.sf.net>
- * Copyright  2005 Marcus Meissner <marcus@jet.franken.de>
+ * Copyright (c) 2001 Lutz Mller <lutz@users.sf.net>
+ * Copyright (c) 2005 Marcus Meissner <marcus@jet.franken.de>
+ * Copyright (c) 2007 Hubert Figuiere <hub@figuiere.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -27,15 +28,31 @@
 #include <sys/types.h>
 #include <utime.h>
 #include <unistd.h>
+#ifdef HAVE_SYS_STATVFS_H
+# include <sys/statvfs.h>
+#endif
+#ifdef HAVE_SYS_PARAM_H
+# include <sys/param.h>
+#endif
+#ifdef HAVE_SYS_VFS_H
+# include <sys/vfs.h>
+#endif
+#ifdef HAVE_SYS_MOUNT_H
+# include <sys/mount.h>
+#endif
+#ifdef HAVE_SYS_STATFS_H
+# include <sys/statfs.h>
+#endif
+
 
 #ifdef HAVE_LIBEXIF
 #include <libexif/exif-data.h>
 #endif
 
-#include <gphoto2-setting.h>
-#include <gphoto2-library.h>
-#include <gphoto2-port.h>
-#include <gphoto2-port-log.h>
+#include <gphoto2/gphoto2-setting.h>
+#include <gphoto2/gphoto2-library.h>
+#include <gphoto2/gphoto2-port.h>
+#include <gphoto2/gphoto2-port-log.h>
 
 #ifdef ENABLE_NLS
 #  include <libintl.h>
@@ -75,6 +92,14 @@ static const struct {
 	{"mrw",  GP_MIME_RAW},
 	{"dng",  GP_MIME_RAW},
 	{"sr2",  GP_MIME_RAW},
+	{"raf",  GP_MIME_RAW},
+	{"erf",  GP_MIME_RAW},
+	{"orf",  GP_MIME_RAW},
+	{"arw",  GP_MIME_RAW},
+	{"pef",  GP_MIME_RAW},
+	{"raw",  GP_MIME_RAW},
+	{"dcr",  GP_MIME_RAW},
+	{"x3f",  GP_MIME_RAW},
 	{"png",  GP_MIME_PNG},
 	{"wav",  GP_MIME_WAV},
 	{NULL, NULL}
@@ -175,10 +200,12 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 			return GP_OK;
 	} else {
 		/* old style access */
-		if (folder[strlen(folder)-1] != '/')
-			sprintf (f, "%s%c", folder, '/');
-		else
-			strcpy (f, folder);
+		if (folder[strlen(folder)-1] != '/') {
+			snprintf (f, sizeof(f), "%s%c", folder, '/');
+		}
+		else {
+			strncpy (f, folder, sizeof(f));
+		}
 	}
 	dir = gp_system_opendir ((char*) f);
 	if (!dir)
@@ -199,7 +226,7 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 				"'%s'..."), f);
 	n = 0;
 	while ((de = gp_system_readdir(dir))) {
-
+		const char * filename = NULL;
 		/* Give some feedback */
 		gp_context_progress_update (context, id, n + 1);
 		gp_context_idle (context);
@@ -208,12 +235,12 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 			return (GP_ERROR_CANCEL);
 		}
 
-		if (strcmp(gp_system_filename(de), "." ) &&
-		    strcmp(gp_system_filename(de), "..")) {
-			sprintf (buf, "%s%s", f, gp_system_filename (de));
-			if (gp_system_is_file (buf) && (get_mime_type (buf)))
-				gp_list_append (list, gp_system_filename (de),
-						NULL);
+		filename = gp_system_filename(de);
+		if (*filename != '.') {
+			snprintf (buf, sizeof(buf), "%s%s", f, filename);
+			if (gp_system_is_file (buf) && (get_mime_type (buf))) {
+				gp_list_append (list, filename,	NULL);
+			}
 		}
 		n++;
 	}
@@ -251,14 +278,16 @@ folder_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 	} else {
 		/* old style access */
 		/* Make sure we have 1 delimiter */
-		if (folder[strlen(folder)-1] != '/')
-			sprintf (f, "%s%c", folder, '/');
-		else
-			strcpy (f, folder);
+		if (folder[strlen(folder)-1] != '/') {
+			snprintf (f, sizeof(f), "%s%c", folder, '/');
+		}
+		else {
+			strncpy (f, folder, sizeof(f));
+		}
 	}
 	dir = gp_system_opendir ((char*) f);
 	if (!dir)
-		return (GP_ERROR);
+		return GP_ERROR;
 	/* Count the files */
 	n = 0;
 	while (gp_system_readdir (dir))
@@ -267,32 +296,33 @@ folder_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 
 	dir = gp_system_opendir (f);
 	if (!dir)
-		return (GP_ERROR);
+		return GP_ERROR;
 	id = gp_context_progress_start (context, n, _("Listing folders in "
 					"'%s'..."), folder);
 	n = 0;
 	while ((de = gp_system_readdir (dir))) {
+		const char * filename = NULL;
+
 		/* Give some feedback */
 		gp_context_progress_update (context, id, n + 1);
 		gp_context_idle (context);
 		if (gp_context_cancel (context) == GP_CONTEXT_FEEDBACK_CANCEL) {
 			gp_system_closedir (dir);
-			return (GP_ERROR_CANCEL);
+			return GP_ERROR_CANCEL;
 		}
-		if ((strcmp (gp_system_filename (de), "." ) != 0) &&
-		    (strcmp (gp_system_filename (de), "..") != 0)) {
-			sprintf (buf, "%s%s", f, gp_system_filename (de));
-
+		filename = gp_system_filename (de);
+		if (*filename != '.') {
+			snprintf (buf, sizeof(buf), "%s%s", f, filename);
+			
 			/* lstat ... do not follow symlinks */
 			if (lstat (buf, &st) != 0) {
 				gp_context_error (context, _("Could not get information "
-						  "about '%s' (%m)."), buf);
-				return (GP_ERROR);
+																		 "about '%s' (%m)."), buf);
+				return GP_ERROR;
+			}
+			if (S_ISDIR (st.st_mode)) {
+				gp_list_append(list,	filename,	NULL);
 			}
-			if (S_ISDIR (st.st_mode))
-				gp_list_append (list,
-						gp_system_filename (de),
-						NULL);
 		}
 		n++;
 	}
@@ -674,6 +704,55 @@ camera_capture (Camera *camera, CameraCaptureType type, CameraFilePath *path,
 }
 #endif
 
+static int
+storage_info_func (CameraFilesystem *fs,
+                CameraStorageInformation **sinfos,
+                int *nrofsinfos,
+                void *data, GPContext *context
+) {
+	Camera 		*camera          = data;
+	GPPortSettings	settings;
+	CameraStorageInformation	*sfs;
+
+#if defined(linux) && defined(HAVE_STATFS)
+	struct	statfs		stfs;
+
+	gp_port_get_settings (camera->port, &settings);
+	if (-1 == statfs (settings.disk.mountpoint, &stfs))
+		return GP_ERROR_NOT_SUPPORTED;
+
+	sfs = malloc (sizeof (CameraStorageInformation));
+	if (!sfs)
+		return GP_ERROR_NO_MEMORY;
+	*sinfos = sfs;
+	*nrofsinfos = 1;
+
+	sfs->fields =	GP_STORAGEINFO_BASE		|
+			GP_STORAGEINFO_DESCRIPTION	|
+			GP_STORAGEINFO_STORAGETYPE	|
+			GP_STORAGEINFO_FILESYSTEMTYPE	|
+			GP_STORAGEINFO_ACCESS		|
+			GP_STORAGEINFO_MAXCAPACITY	|
+			GP_STORAGEINFO_FREESPACEKBYTES;
+	;
+	strcpy (sfs->basedir, "/");
+	strcpy (sfs->description, "Directory Driver");
+	sfs->type		= GP_STORAGEINFO_ST_REMOVABLE_RAM;
+	sfs->fstype		= GP_STORAGEINFO_FST_GENERICHIERARCHICAL;
+	sfs->access		= GP_STORAGEINFO_AC_READWRITE;
+	if (stfs.f_bsize >= 1024) {
+		sfs->capacitykbytes	= stfs.f_blocks*(stfs.f_bsize/1024);
+		sfs->freekbytes		= stfs.f_bavail*(stfs.f_bsize/1024);
+	} else {
+		sfs->capacitykbytes	= stfs.f_blocks/(1024/stfs.f_bsize);
+		sfs->freekbytes		= stfs.f_bavail/(1024/stfs.f_bsize);
+	}
+	return GP_OK;
+#endif
+
+	return GP_ERROR_NOT_SUPPORTED;
+} 
+
 static CameraFilesystemFuncs fsfuncs = {
 	.file_list_func = file_list_func,
 	.folder_list_func = folder_list_func,
@@ -684,6 +763,7 @@ static CameraFilesystemFuncs fsfuncs = {
 	.del_file_func = delete_file_func,
 	.make_dir_func = make_dir_func,
 	.remove_dir_func = remove_dir_func,
+	.storage_info_func = storage_info_func,
 };
 
 int
